@@ -2,7 +2,9 @@
 """Module to scrap book information from goodreads
 """
 
-__version__ = '0.1'
+__version__ = '0.2'
+
+__description__ = 'Module to scrap book information from goodreads'
 
 import os
 import time
@@ -10,8 +12,8 @@ import re
 import json
 import traceback
 import jellyfish
-import pandas as pd
 import threading as th
+import requests
 from typing import List
 from bs4 import BeautifulSoup
 from selenium import webdriver
@@ -26,11 +28,6 @@ from .review import Review
 path = os.path.dirname(os.path.realpath(__file__))
 
 
-"""
-TODO:
-	- Criar/permitir uso de modelo de aprendizagem profunda para analisar reviews
-"""
-
 
 
 
@@ -39,20 +36,19 @@ def create_driver(args)->webdriver:
 	driver = None
 	log(args,f'Creating driver')
 	try:
-		log(args,f'Chrome')
-		opg = webdriver.ChromeOptions()
-		opg.add_argument('--window-size=1440,900')
-		opg.add_argument('--headless')
-		driver = webdriver.Chrome(options=opg)
-		
+		log(args,f'Firefox')
+		opf = webdriver.FirefoxOptions()
+		opf.add_argument('--window-size=1440,900')
+		opf.add_argument('--headless')
+		driver = webdriver.Firefox(options=opf)
 	except:
 		log(args,f'Could not create Chrome driver')
 		try:
-			log(args,f'Firefox')
-			opf = webdriver.FirefoxOptions()
-			opf.add_argument('--window-size=1440,900')
-			opf.add_argument('--headless')
-			driver = webdriver.Firefox(options=opf)
+			log(args,f'Chrome')
+			opg = webdriver.ChromeOptions()
+			opg.add_argument('--window-size=1440,900')
+			opg.add_argument('--headless')
+			driver = webdriver.Chrome(options=opg)
 		except:
 			log(args,f'Could not create Firefox driver')
 			driver = None
@@ -64,6 +60,7 @@ def create_driver(args)->webdriver:
 		error(args,f'Could not create driver. Please read documentation to install proper dependencies.')
 	return driver
 
+
 def driver_wait_element_to_be_clickable(driver:webdriver,xpath:str,timeout:int=1):
 	'''Waits until timeout for element in XPATH to become clickable'''
 	WebDriverWait(driver, timeout).until(EC.element_to_be_clickable((By.XPATH,xpath)))
@@ -77,9 +74,7 @@ def driver_wait_element(driver,xpath:str,timeout:int=1,exist:bool=True):
 
 
 
-def create_dataset(header,list)->pd.DataFrame:
-	"""Creates a dataset from a list of already processed into strings of dataset lines"""
-	return pd.DataFrame(list,columns=header)
+
 
 
 def search_book_option(args):
@@ -374,7 +369,6 @@ def get_author_works(args,works_url:str,max:int=None)->List[str]:
 
 def scrape_author_page(args,html_page:str)->Author:
 	"""Scrapes an author's page for info such as name, birthday, average score, number of reviews and others"""
-	# FIXME: if elses para tudo pq pode n ter nada - ex. Sir kazzio
 	log(args,f"Scraping author's page for info")
 	page = BeautifulSoup(html_page,features='lxml')
 
@@ -428,10 +422,30 @@ def scrape_author_page(args,html_page:str)->Author:
 		   author_nratings,author_nreviews,author_nUniqueWorks,author_works)
 
 
-def loaded_reviews_page(html:str)->bool:
-	"""Checks if the reviews page is loaded"""
-	#FIXME
-	return True
+def load_reviews_page(args,driver)->bool:
+	"""Waits for review page to be loaded, refreshes some times if stuck"""
+	loaded = False 
+	tries = 5
+	while not loaded and tries > 0:
+		try:
+			log(args,f'Waiting for page to load...')
+			# Wait for reviews to appear
+			driver_wait_element(driver,"//div[@class='ReviewsList']",20)
+			# Check if something is still loading
+			driver_wait_element(driver,"//div[@class='LoadingCard']",20,False)
+			if re.search('ROOT_QUERY',driver.page_source):
+				loaded = True
+			else:
+				log(args,f'Refreshing page...')
+				driver.get(driver.current_url)
+				time.sleep(1)
+			tries-= 1
+		except:
+			log(args,f'Refreshing page...')
+			driver.get(driver.current_url)
+			time.sleep(1)
+			tries-= 1
+	return loaded
 
 def get_book_reviews_page(args,driver)->str:
 	"""Return the book reviews page (if book flags active)"""
@@ -451,11 +465,7 @@ def get_book_reviews_page(args,driver)->str:
 		if id:		
 			log(args,f"Trying to get reviews page")
 			driver.get(f'https://www.goodreads.com/book/show/{id}/reviews')
-			# Wait for reviews to appear
-			driver_wait_element(driver,"//div[@class='ReviewsList']",100000)
-			# Check if something is still loading
-			driver_wait_element(driver,"//div[@class='LoadingCard']",100000,False)
-			if not loaded_reviews_page(driver.page_source):
+			if not load_reviews_page(args,driver):
 				error(args,'Could not get proper reviews page')
 			else:
 				r = driver.page_source
@@ -470,6 +480,7 @@ def scrape_reviews_page(args,html_page:str,lower_limit:int=None,higher_limit:int
 	log(args,f"Scraping reviews's page for info")
 
 	page = BeautifulSoup(html_page,features='lxml')
+	book_name = page.find('a',{'data-testid':'title'}).get_text()
 	div_reviews_list = page.find('div',{'class':'ReviewsList'})
 	reviews = div_reviews_list.find_all('article',{'class':'ReviewCard'})
 	# Change range of reviews to gather
@@ -486,7 +497,7 @@ def scrape_reviews_page(args,html_page:str,lower_limit:int=None,higher_limit:int
 			review_score = re.search(r'Rating (\d+) out of \d+',review_score_elem['aria-label']).group(1)
 		review_description_card = review.find('section',{'class','ReviewText__content'})
 		review_description = review_description_card.find('span').get_text().strip()
-		rev = Review(reviewer_id,reviewer_name,review_score,review_description)
+		rev = Review(book_name,reviewer_id,reviewer_name,review_score,review_description)
 		reviews_list.append(rev)
 	log(args,f"Scraping finished")
 	return reviews_list
@@ -496,11 +507,21 @@ def scrape_reviews_page(args,html_page:str,lower_limit:int=None,higher_limit:int
 def get_review_page_stats(page:str)->List[int]:
 	"""Recovers simple stats about review page, such as number of reviews and range shown"""
 	soup = BeautifulSoup(page,features='lxml')
-	number_reviews_elem = soup.find('div',{'class':'ReviewsList__listContext'}).find('span')
-	number_reviews_info = re.search(r'(\d+((,|\.)\d+)?)[^\d]*(\d+((,|\.)\d+)?)[^\d]*(\d+((,|\.)\d+)?)',number_reviews_elem.get_text().strip())
-	n_reviews = int(re.sub(r'(.*),|\.(.*)',r'\1\2',number_reviews_info.group(7)))
-	lower_review = int(re.sub(r'(.*),|\.(.*)',r'\1\2',number_reviews_info.group(1)))
-	higher_review = int(re.sub(r'(.*),|\.(.*)',r'\1\2',number_reviews_info.group(4)))
+	try:
+		number_reviews_elem = soup.find('div',{'class':'ReviewsList__listContext'}).find('span')
+		number_reviews_info = re.search(r'(\d+((,|\.)\d+)?)[^\d]*(\d+((,|\.)\d+)?)[^\d]*(\d+((,|\.)\d+)?)',number_reviews_elem.get_text().strip())
+		if len(number_reviews_info.groups()) >= 7:
+			n_reviews = int(re.sub(r'(.*),|\.(.*)',r'\1\2',number_reviews_info.group(7)))
+			lower_review = int(re.sub(r'(.*),|\.(.*)',r'\1\2',number_reviews_info.group(1)))
+			higher_review = int(re.sub(r'(.*),|\.(.*)',r'\1\2',number_reviews_info.group(4)))
+		else :
+			n_reviews = int(re.sub(r'(.*),|\.(.*)',r'\1\2',number_reviews_info.group(4)))
+			lower_review = int(re.sub(r'(.*),|\.(.*)',r'\1\2',number_reviews_info.group(1)))
+			higher_review = int(re.sub(r'(.*),|\.(.*)',r'\1\2',number_reviews_info.group(4)))
+	except:
+		n_reviews = 0
+		lower_review = 0
+		higher_review = 0
 	return [n_reviews,lower_review,higher_review]
 
 
@@ -644,10 +665,10 @@ def goodreadsscraper():
 						b = scrape_book_page(book,prettify_html(res))
 						results.append({'out':book.output,'result':b.__str__(True if args.verbose else False)})
 					
-					if(book.reviews_simple or book.reviews):
+					if(book.reviews_simple or book.reviews and res):
 						reviews = scrape_reviews(book,driver)
 						if reviews:
-							delim = '#;#'
+							delim = ';'
 							# Allow for multiple review pages to be written in the main review output
 							if not book.reviews_output:
 								if first_write:
@@ -656,7 +677,8 @@ def goodreadsscraper():
 								else:
 									write_reviews(args,reviews,first_write,True,delim)
 							else:
-								data = costum_csv([review.dataset_line_str(delim) for review in reviews],reviews[0].header_str(delim))
+								df = create_dataset(reviews[0].header(),[review.dataset_line() for review in reviews])
+								data = df.to_csv(None,sep=delim,columns=reviews[0].header(),quoting=1)
 								out = open(book.reviews_output,'w') if book.reviews_output else None
 								results_reviews.append({'out' : out, 'result':data})
 
